@@ -3,7 +3,7 @@ package Make;
 use strict;
 use warnings;
 
-our $VERSION = '2.007';
+our $VERSION = '2.008';
 
 use Carp qw(confess croak);
 use Config;
@@ -496,6 +496,31 @@ sub parse_cmdline {
     return \%parsed;
 }
 
+## no critic (BuiltinFunctions::RequireBlockMap)
+my %NAME_QUOTING     = map +( $_ => sprintf "%%%02x", ord $_ ), qw(% :);
+my $NAME_QUOTE_CHARS = join '', '[', ( map quotemeta, sort keys %NAME_QUOTING ), ']';
+
+sub name_encode {
+    join ':', map {
+        my $s = $_;
+        $s =~ s/($NAME_QUOTE_CHARS)/$NAME_QUOTING{$1}/gs;
+        $s
+    } @{ $_[0] };
+}
+
+sub name_decode {
+    my ($s) = @_;
+    [
+        map {
+            my $s = $_;
+            $s =~ s/%(..)/chr hex $1/ges;
+            $s
+        } split ':',
+        $_[0]
+    ];
+}
+## use critic
+
 ## no critic (Subroutines::ProhibitBuiltinHomonyms)
 sub exec {
     my ( $self, $line ) = @_;
@@ -531,6 +556,39 @@ sub parse_args {
 }
 ## use critic
 
+sub as_graph {
+    my ($self) = @_;
+    require Graph;
+    my $g = Graph->new;
+    my %recipe_cache;
+    for my $target ( sort $self->targets ) {
+        my $node_name = name_encode( [ 'target', $target ] );
+        $g->add_vertex($node_name);
+        my $make_target = $self->target($target);
+        my $rule_no     = 0;
+        for my $rule ( @{ $make_target->rules } ) {
+            my $recipe = $rule->recipe;
+            my $rule_id
+                = $recipe_cache{$recipe} || ( $recipe_cache{$recipe} = name_encode( [ 'rule', $target, $rule_no ] ) );
+            $g->set_vertex_attributes(
+                $rule_id,
+                {
+                    recipe     => $recipe,
+                    recipe_raw => $rule->recipe_raw,
+                }
+            );
+            $g->add_edge( $node_name, $rule_id );
+            for my $dep ( @{ $rule->prereqs } ) {
+                my $dep_node = name_encode( [ 'target', $dep ] );
+                $g->add_vertex($dep_node);
+                $g->add_edge( $rule_id, $dep_node );
+            }
+            $rule_no++;
+        }
+    }
+    return $g;
+}
+
 sub apply {
     my ( $self, $method, @args ) = @_;
     $self->NextPass;
@@ -775,8 +833,37 @@ Returns an array-ref of the packages to search for macro functions.
 
 Returns a hash-ref of the L</FSFunctionMap>.
 
+=head2 as_graph
+
+Returns a L<Graph::Directed> object representing the makefile. The
+vertices are named either C<target:name> (representing L<Make::Target>s)
+or C<rule:name:rule_index> (representing L<Make::Rule>s). The names
+encoded with L</name_encode>. Rules are named according to the first
+(alphabetically) target they are attached to.
+
+The rule vertices have attributes with the same values as the
+L<Make::Rule> attributes:
+
+=over
+
+=item recipe
+
+=item recipe_raw
+
+=back
+
 =head1 FUNCTIONS
 
+=head2 name_encode
+
+=head2 name_decode
+
+    my $encoded = Make::name_encode([ 'target', 'all' ]);
+    my $tuple = Make::name_decode($encoded); # [ 'target', 'all' ]
+
+Uses C<%>-encoding and -decoding to allow C<%> and C<:> characters in
+components without problems.
+
 =head2 parse_makefile
 
 Given a file-handle, returns array-ref of Abstract Syntax-Tree (AST)
